# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 50.4.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class ReportInputItemV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'link_url': 'str',
        'screenshot_url': 'str',
        'worksheet_id': 'str'
    }

    attribute_map = {
        'link_url': 'linkURL',
        'screenshot_url': 'screenshotURL',
        'worksheet_id': 'worksheetId'
    }

    def __init__(self, link_url=None, screenshot_url=None, worksheet_id=None):
        """
        ReportInputItemV1 - a model defined in Swagger
        """

        self._link_url = None
        self._screenshot_url = None
        self._worksheet_id = None

        if link_url is not None:
          self.link_url = link_url
        if screenshot_url is not None:
          self.screenshot_url = screenshot_url
        if worksheet_id is not None:
          self.worksheet_id = worksheet_id

    @property
    def link_url(self):
        """
        Gets the link_url of this ReportInputItemV1.
        Link URL to use for this Worksheet. If empty, no link will be included

        :return: The link_url of this ReportInputItemV1.
        :rtype: str
        """
        return self._link_url

    @link_url.setter
    def link_url(self, link_url):
        """
        Sets the link_url of this ReportInputItemV1.
        Link URL to use for this Worksheet. If empty, no link will be included

        :param link_url: The link_url of this ReportInputItemV1.
        :type: str
        """

        self._link_url = link_url

    @property
    def screenshot_url(self):
        """
        Gets the screenshot_url of this ReportInputItemV1.
        URL from which to generate a screenshot for this Worksheet. If empty, no screenshot will be included

        :return: The screenshot_url of this ReportInputItemV1.
        :rtype: str
        """
        return self._screenshot_url

    @screenshot_url.setter
    def screenshot_url(self, screenshot_url):
        """
        Sets the screenshot_url of this ReportInputItemV1.
        URL from which to generate a screenshot for this Worksheet. If empty, no screenshot will be included

        :param screenshot_url: The screenshot_url of this ReportInputItemV1.
        :type: str
        """

        self._screenshot_url = screenshot_url

    @property
    def worksheet_id(self):
        """
        Gets the worksheet_id of this ReportInputItemV1.
        Worksheet ID to include in the report

        :return: The worksheet_id of this ReportInputItemV1.
        :rtype: str
        """
        return self._worksheet_id

    @worksheet_id.setter
    def worksheet_id(self, worksheet_id):
        """
        Sets the worksheet_id of this ReportInputItemV1.
        Worksheet ID to include in the report

        :param worksheet_id: The worksheet_id of this ReportInputItemV1.
        :type: str
        """
        if worksheet_id is None:
            raise ValueError("Invalid value for `worksheet_id`, must not be `None`")

        self._worksheet_id = worksheet_id

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, ReportInputItemV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



import pytest
from seeq import spy
import ipywidgets as ipw
import logging


@pytest.mark.system
def test_log_handler_trim_lines():
    widget_log_handler = spy.widgets.WidgetLogHandler()
    widget_log_handler.delimiter = '\n'
    widget_log_handler.maximum_lines = 3
    test_string = 'Line 1\nLine 2\nLine 3\nLine 4'
    trim_start = widget_log_handler._trim_lines(test_string, 'start')
    assert trim_start.split(widget_log_handler.delimiter) == test_string.split(widget_log_handler.delimiter)[2:]

    trim_end = widget_log_handler._trim_lines(test_string, 'end')
    assert trim_end.split(widget_log_handler.delimiter) == test_string.split(widget_log_handler.delimiter)[:-2]


@pytest.mark.system
def test_log_handler():
    log_output_widget = ipw.Textarea(
        value='Line 1\nLine 2\nLine 3'
    )
    widget_log_handler = spy.widgets.WidgetLogHandler()
    # a fake log record
    test_message = 'Test Message'
    test_record = logging.LogRecord(name='test_logger',
                                    level=logging.DEBUG,
                                    pathname='test_file_path',
                                    lineno=100,
                                    msg=test_message,
                                    args=[],
                                    exc_info=None)
    widget_log_handler.set_widget(log_output_widget, concat='append', delimiter='\n')
    widget_log_handler.emit(test_record)
    assert log_output_widget.value.split('\n')[-1] == test_message

    widget_log_handler.concat = 'prepend'
    widget_log_handler.emit(test_record)
    assert log_output_widget.value.split('\n')[0] == test_message

    assert len(log_output_widget.value.split('\n')) == 5



# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 50.4.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import models into sdk package
from .models.access_key_input_v1 import AccessKeyInputV1
from .models.access_key_output_list_v1 import AccessKeyOutputListV1
from .models.access_key_output_v1 import AccessKeyOutputV1
from .models.ace_input_v1 import AceInputV1
from .models.ace_output_v1 import AceOutputV1
from .models.acl_input_v1 import AclInputV1
from .models.acl_output_v1 import AclOutputV1
from .models.activity_graph_edge_output_v1 import ActivityGraphEdgeOutputV1
from .models.activity_graph_output_v1 import ActivityGraphOutputV1
from .models.activity_output_v1 import ActivityOutputV1
from .models.administrator_contact_information_v1 import AdministratorContactInformationV1
from .models.agent_key_output_v1 import AgentKeyOutputV1
from .models.agent_status_v1 import AgentStatusV1
from .models.ancillary_input_v1 import AncillaryInputV1
from .models.ancillary_item_input_v1 import AncillaryItemInputV1
from .models.ancillary_item_output_v1 import AncillaryItemOutputV1
from .models.ancillary_output_v1 import AncillaryOutputV1
from .models.annotation_input_v1 import AnnotationInputV1
from .models.annotation_interest_input_v1 import AnnotationInterestInputV1
from .models.annotation_interest_output_v1 import AnnotationInterestOutputV1
from .models.annotation_list_output_v1 import AnnotationListOutputV1
from .models.annotation_output_v1 import AnnotationOutputV1
from .models.archive_signal_output_v1 import ArchiveSignalOutputV1
from .models.asset_batch_input_v1 import AssetBatchInputV1
from .models.asset_input_v1 import AssetInputV1
from .models.asset_output_v1 import AssetOutputV1
from .models.asset_tree_batch_input_v1 import AssetTreeBatchInputV1
from .models.asset_tree_output_v1 import AssetTreeOutputV1
from .models.asset_tree_single_input_v1 import AssetTreeSingleInputV1
from .models.audit_output_list_v1 import AuditOutputListV1
from .models.audit_output_v1 import AuditOutputV1
from .models.auth_input_v1 import AuthInputV1
from .models.auth_providers_output_v1 import AuthProvidersOutputV1
from .models.cache_info_v1 import CacheInfoV1
from .models.calculated_item_output_v1 import CalculatedItemOutputV1
from .models.capsule_v1 import CapsuleV1
from .models.capsules_input_v1 import CapsulesInputV1
from .models.capsules_output_v1 import CapsulesOutputV1
from .models.channel_output_v1 import ChannelOutputV1
from .models.condition_batch_input_v1 import ConditionBatchInputV1
from .models.condition_input_v1 import ConditionInputV1
from .models.condition_output_v1 import ConditionOutputV1
from .models.configuration_input_v1 import ConfigurationInputV1
from .models.configuration_option_input_v1 import ConfigurationOptionInputV1
from .models.configuration_option_output_simple_v1 import ConfigurationOptionOutputSimpleV1
from .models.configuration_option_output_v1 import ConfigurationOptionOutputV1
from .models.configuration_output_v1 import ConfigurationOutputV1
from .models.connection_status_v1 import ConnectionStatusV1
from .models.content_input_v1 import ContentInputV1
from .models.content_output_v1 import ContentOutputV1
from .models.content_with_metadata_list_output_v1 import ContentWithMetadataListOutputV1
from .models.content_with_metadata_output_v1 import ContentWithMetadataOutputV1
from .models.csv_datafile_output_v1 import CsvDatafileOutputV1
from .models.datafile_input_v1 import DatafileInputV1
from .models.datafile_output_v1 import DatafileOutputV1
from .models.datasource_clean_up_input_v1 import DatasourceCleanUpInputV1
from .models.datasource_clean_up_output_v1 import DatasourceCleanUpOutputV1
from .models.datasource_input_v1 import DatasourceInputV1
from .models.datasource_output_list_v1 import DatasourceOutputListV1
from .models.datasource_output_v1 import DatasourceOutputV1
from .models.datasource_preview_v1 import DatasourcePreviewV1
from .models.datasource_statistics_v1 import DatasourceStatisticsV1
from .models.datasource_status_output_v1 import DatasourceStatusOutputV1
from .models.datasource_status_v1 import DatasourceStatusV1
from .models.date_range_input_v1 import DateRangeInputV1
from .models.date_range_output_v1 import DateRangeOutputV1
from .models.document_backup_output_v1 import DocumentBackupOutputV1
from .models.export_item_v1 import ExportItemV1
from .models.export_items_output_v1 import ExportItemsOutputV1
from .models.export_items_v1 import ExportItemsV1
from .models.export_preview_list_v1 import ExportPreviewListV1
from .models.export_preview_v1 import ExportPreviewV1
from .models.external_tool_input_v1 import ExternalToolInputV1
from .models.external_tool_output_v1 import ExternalToolOutputV1
from .models.folder_content_output_v1 import FolderContentOutputV1
from .models.folder_input_v1 import FolderInputV1
from .models.folder_navigation_output_v1 import FolderNavigationOutputV1
from .models.folder_output_list_v1 import FolderOutputListV1
from .models.folder_output_v1 import FolderOutputV1
from .models.formula_compile_output_v1 import FormulaCompileOutputV1
from .models.formula_doc_example_input_v1 import FormulaDocExampleInputV1
from .models.formula_doc_example_list_input_v1 import FormulaDocExampleListInputV1
from .models.formula_doc_input_v1 import FormulaDocInputV1
from .models.formula_doc_keyword_list_input_v1 import FormulaDocKeywordListInputV1
from .models.formula_doc_output_v1 import FormulaDocOutputV1
from .models.formula_doc_summaries_output_v1 import FormulaDocSummariesOutputV1
from .models.formula_doc_summary_output_v1 import FormulaDocSummaryOutputV1
from .models.formula_error_output_v1 import FormulaErrorOutputV1
from .models.formula_example_output_v1 import FormulaExampleOutputV1
from .models.formula_input_v1 import FormulaInputV1
from .models.formula_item_input_v1 import FormulaItemInputV1
from .models.formula_item_output_v1 import FormulaItemOutputV1
from .models.formula_log_entry import FormulaLogEntry
from .models.formula_log_entry_details import FormulaLogEntryDetails
from .models.formula_log_v1 import FormulaLogV1
from .models.formula_package_input_v1 import FormulaPackageInputV1
from .models.formula_package_output_v1 import FormulaPackageOutputV1
from .models.formula_parameter_input_v1 import FormulaParameterInputV1
from .models.formula_parameter_output_v1 import FormulaParameterOutputV1
from .models.formula_run_input_v1 import FormulaRunInputV1
from .models.formula_run_output_v1 import FormulaRunOutputV1
from .models.formula_token import FormulaToken
from .models.formula_upgrade_change_v1 import FormulaUpgradeChangeV1
from .models.formula_upgrade_output_v1 import FormulaUpgradeOutputV1
from .models.function_input_v1 import FunctionInputV1
from .models.function_parameter_output_v1 import FunctionParameterOutputV1
from .models.function_variant_output_v1 import FunctionVariantOutputV1
from .models.gauge_datum_v1 import GaugeDatumV1
from .models.generic_table_output_v1 import GenericTableOutputV1
from .models.get_channels_output_v1 import GetChannelsOutputV1
from .models.get_content_items_output_v1 import GetContentItemsOutputV1
from .models.get_date_ranges_output_v1 import GetDateRangesOutputV1
from .models.get_external_tools_output_v1 import GetExternalToolsOutputV1
from .models.get_jobs_output_v1 import GetJobsOutputV1
from .models.get_metrics_output_v1 import GetMetricsOutputV1
from .models.get_projects_output_v1 import GetProjectsOutputV1
from .models.get_request_output_v1 import GetRequestOutputV1
from .models.get_requests_output_v1 import GetRequestsOutputV1
from .models.get_sample_output_v1 import GetSampleOutputV1
from .models.get_samples_output_v1 import GetSamplesOutputV1
from .models.get_signals_output_v1 import GetSignalsOutputV1
from .models.identity_mapping_list_v1 import IdentityMappingListV1
from .models.identity_mapping_v1 import IdentityMappingV1
from .models.identity_preview_list_v1 import IdentityPreviewListV1
from .models.identity_preview_v1 import IdentityPreviewV1
from .models.item_ancillary_output_v1 import ItemAncillaryOutputV1
from .models.item_batch_output_v1 import ItemBatchOutputV1
from .models.item_dependency_output_v1 import ItemDependencyOutputV1
from .models.item_id_list_input_v1 import ItemIdListInputV1
from .models.item_output_v1 import ItemOutputV1
from .models.item_parameter_of_output_v1 import ItemParameterOfOutputV1
from .models.item_preview_list_v1 import ItemPreviewListV1
from .models.item_preview_v1 import ItemPreviewV1
from .models.item_preview_with_assets_v1 import ItemPreviewWithAssetsV1
from .models.item_search_preview_list_v1 import ItemSearchPreviewListV1
from .models.item_search_preview_paginated_list_v1 import ItemSearchPreviewPaginatedListV1
from .models.item_search_preview_v1 import ItemSearchPreviewV1
from .models.item_update_output_v1 import ItemUpdateOutputV1
from .models.item_user_attributes_input_v1 import ItemUserAttributesInputV1
from .models.item_user_attributes_output_v1 import ItemUserAttributesOutputV1
from .models.item_with_swap_pairs_v1 import ItemWithSwapPairsV1
from .models.job_output_v1 import JobOutputV1
from .models.license_importer_output_v1 import LicenseImporterOutputV1
from .models.license_status_output_v1 import LicenseStatusOutputV1
from .models.licensed_feature_status_output_v1 import LicensedFeatureStatusOutputV1
from .models.log_message import LogMessage
from .models.meter_datum_v1 import MeterDatumV1
from .models.monitor_definition_output_v1 import MonitorDefinitionOutputV1
from .models.monitor_definitions_output_v1 import MonitorDefinitionsOutputV1
from .models.monitor_input_v1 import MonitorInputV1
from .models.monitor_output_v1 import MonitorOutputV1
from .models.monitor_values import MonitorValues
from .models.monitors_output_v1 import MonitorsOutputV1
from .models.optional_report_input_v1 import OptionalReportInputV1
from .models.parameter_doc_output_v1 import ParameterDocOutputV1
from .models.permissions_v1 import PermissionsV1
from .models.plugin_output_list_v1 import PluginOutputListV1
from .models.plugin_output_v1 import PluginOutputV1
from .models.priority_v1 import PriorityV1
from .models.progress_information_output_v1 import ProgressInformationOutputV1
from .models.project_input_v1 import ProjectInputV1
from .models.project_output_v1 import ProjectOutputV1
from .models.property_href_output_v1 import PropertyHrefOutputV1
from .models.property_input_v1 import PropertyInputV1
from .models.property_output_v1 import PropertyOutputV1
from .models.put_samples_input_v1 import PutSamplesInputV1
from .models.put_samples_output_v1 import PutSamplesOutputV1
from .models.put_scalars_input_v1 import PutScalarsInputV1
from .models.put_signals_input_v1 import PutSignalsInputV1
from .models.put_user_groups_input_v1 import PutUserGroupsInputV1
from .models.regression_output_v1 import RegressionOutputV1
from .models.report_input_item_v1 import ReportInputItemV1
from .models.report_input_v1 import ReportInputV1
from .models.request_output_v1 import RequestOutputV1
from .models.sample_input_v1 import SampleInputV1
from .models.sample_output_v1 import SampleOutputV1
from .models.scalar_input_v1 import ScalarInputV1
from .models.scalar_property_v1 import ScalarPropertyV1
from .models.scalar_value_output_v1 import ScalarValueOutputV1
from .models.screenshot_output_v1 import ScreenshotOutputV1
from .models.see_also_doc_output_v1 import SeeAlsoDocOutputV1
from .models.send_logs_input_v1 import SendLogsInputV1
from .models.series_batch_input_v1 import SeriesBatchInputV1
from .models.series_input_v1 import SeriesInputV1
from .models.series_output_v1 import SeriesOutputV1
from .models.series_sample_v1 import SeriesSampleV1
from .models.series_samples_input_v1 import SeriesSamplesInputV1
from .models.series_samples_output_v1 import SeriesSamplesOutputV1
from .models.server_load_output_v1 import ServerLoadOutputV1
from .models.server_spec_output_v1 import ServerSpecOutputV1
from .models.server_status_output_v1 import ServerStatusOutputV1
from .models.signal_input_v1 import SignalInputV1
from .models.signal_output_v1 import SignalOutputV1
from .models.signal_with_id_input_v1 import SignalWithIdInputV1
from .models.status_message_base import StatusMessageBase
from .models.subscription_input_v1 import SubscriptionInputV1
from .models.subscription_output_v1 import SubscriptionOutputV1
from .models.subscription_parameter_output_v1 import SubscriptionParameterOutputV1
from .models.subscription_update_input_v1 import SubscriptionUpdateInputV1
from .models.supported_units_output_v1 import SupportedUnitsOutputV1
from .models.swap_input_v1 import SwapInputV1
from .models.swap_option_list_v1 import SwapOptionListV1
from .models.swap_option_v1 import SwapOptionV1
from .models.sync_progress import SyncProgress
from .models.table_column_output_v1 import TableColumnOutputV1
from .models.table_output_v1 import TableOutputV1
from .models.threshold_metric_input_v1 import ThresholdMetricInputV1
from .models.threshold_metric_output_v1 import ThresholdMetricOutputV1
from .models.threshold_output_v1 import ThresholdOutputV1
from .models.timer_datum_v1 import TimerDatumV1
from .models.tree_item_output_v1 import TreeItemOutputV1
from .models.treemap_item_output_v1 import TreemapItemOutputV1
from .models.treemap_output_v1 import TreemapOutputV1
from .models.user_group_input_v1 import UserGroupInputV1
from .models.user_group_output_v1 import UserGroupOutputV1
from .models.user_group_with_id_input_v1 import UserGroupWithIdInputV1
from .models.user_input_v1 import UserInputV1
from .models.user_output_list_v1 import UserOutputListV1
from .models.user_output_v1 import UserOutputV1
from .models.user_password_input_v1 import UserPasswordInputV1
from .models.workbench_item_output_list_v1 import WorkbenchItemOutputListV1
from .models.workbench_search_result_preview_v1 import WorkbenchSearchResultPreviewV1
from .models.workbook_input_v1 import WorkbookInputV1
from .models.workbook_output_list_v1 import WorkbookOutputListV1
from .models.workbook_output_v1 import WorkbookOutputV1
from .models.worksheet_input_v1 import WorksheetInputV1
from .models.worksheet_output_list_v1 import WorksheetOutputListV1
from .models.worksheet_output_v1 import WorksheetOutputV1
from .models.workstep_input_v1 import WorkstepInputV1
from .models.workstep_output_v1 import WorkstepOutputV1

# import apis into sdk package
from .apis.access_keys_api import AccessKeysApi
from .apis.agents_api import AgentsApi
from .apis.ancillaries_api import AncillariesApi
from .apis.annotations_api import AnnotationsApi
from .apis.assets_api import AssetsApi
from .apis.audit_api import AuditApi
from .apis.auth_api import AuthApi
from .apis.conditions_api import ConditionsApi
from .apis.content_api import ContentApi
from .apis.datafiles_api import DatafilesApi
from .apis.datasources_api import DatasourcesApi
from .apis.export_api import ExportApi
from .apis.folders_api import FoldersApi
from .apis.formulas_api import FormulasApi
from .apis.items_api import ItemsApi
from .apis.jobs_api import JobsApi
from .apis.logs_api import LogsApi
from .apis.metrics_api import MetricsApi
from .apis.monitors_api import MonitorsApi
from .apis.networks_api import NetworksApi
from .apis.plugins_api import PluginsApi
from .apis.projects_api import ProjectsApi
from .apis.reports_api import ReportsApi
from .apis.requests_api import RequestsApi
from .apis.sample_series_api import SampleSeriesApi
from .apis.scalars_api import ScalarsApi
from .apis.signals_api import SignalsApi
from .apis.subscriptions_api import SubscriptionsApi
from .apis.system_api import SystemApi
from .apis.tables_api import TablesApi
from .apis.trees_api import TreesApi
from .apis.user_groups_api import UserGroupsApi
from .apis.users_api import UsersApi
from .apis.workbooks_api import WorkbooksApi

# import ApiClient
from .api_client import ApiClient

from .configuration import Configuration

configuration = Configuration()



import json
import logging
import os
import pkg_resources
import re
import requests
import urllib3

from typing import Optional

import pandas as pd

import seeq
from seeq.base import system
from seeq.sdk import *
from seeq.sdk.rest import ApiException
from seeq import spy

from . import _common, _config, _url
from ._common import Status

from urllib3.connectionpool import MaxRetryError, SSLError

client: Optional[ApiClient] = None
user: Optional[UserOutputV1] = None
server_version: Optional[str] = None
supported_units: Optional[set] = None
corporate_folder: Optional[FolderOutputV1] = None

https_verify_ssl = True
https_key_file = None
https_cert_file = None

AUTOMATIC_PROXY_DETECTION = '__auto__'


def login(username=None, password=None, *, access_key=None, url=None, directory='Seeq', ignore_ssl_errors=False,
          proxy=AUTOMATIC_PROXY_DETECTION, credentials_file=None, auth_token=None, force=True, quiet=False,
          status=None, auth_provider=None):
    """
    Establishes a connection with Seeq Server and logs in with a set of
    credentials. At least one set of credentials must be provided.
    Applicable credential sets are:
        - username + password (where username is in "Seeq" user directory)
        - username + password + directory
        - access_key + password
        - credentials_file (where username is in "Seeq" user directory)
        - credentials_file + directory

    Parameters
    ----------
    username : str, optional
        Username for login purposes. See credentials_file argument for
        alternative.

    password : str, optional
        Password for login purposes. See credentials_file argument for
        alternative.

    access_key: str, optional
        Access Key for login purposes. Access Keys are created by individual
        users via the Seeq user interface in the upper-right user profile
        menu. An Access Key has an associated password that is presented
        to the user (once!) upon creation of the Access Key, and it must be
        supplied via the "password" argument. The "directory" argument must
        NOT be supplied.

    url : str, default 'http://localhost:34216'
        Seeq Server url. You can copy this from your browser and cut off
        everything to the right of the port (if present).
        E.g. https://myseeqserver:34216

    directory : str, default 'Seeq'
        The authentication directory to use. You must be able to supply a
        username/password, so some passwordless Windows Authentication
        (NTLM) scenarios will not work. OpenID Connect is also not
        supported. If you need to use such authentication schemes, set up
        a Seeq Data Lab server.

    ignore_ssl_errors : bool, default False
        If True, SSL certificate validation errors are ignored. Use this
        if you're in a trusted network environment but Seeq Server's SSL
        certificate is not from a common root authority.

    proxy : str, default '__auto__'
        Specifies the proxy server to use for all requests. The default
        value is "__auto__", which examines the standard HTTP_PROXY and
        HTTPS_PROXY environment variables. If you specify None for this
        parameter, no proxy server will be used.

    credentials_file : str, optional
        Reads username and password from the specified file. If specified, the
        file should be plane text and contain two lines, the first line being
        the username, the second being the user's password.

    force : str, default True
        If True, re-logs in even if already already logged in. If False, skips
        login if already logged in. You should include a spy.login(force=False)
        cell if you are creating example notebooks that may be used in Jupyter
        environments like Anaconda, AWS SageMaker or Azure Notebooks.)

    quiet : bool, default False
        If True, suppresses progress output.

    status : spy.Status, optional
        If supplied, this Status object will be updated as the command
        progresses.

    auth_provider : str, deprecated
        Use directory instead.

    auth_token : str, deprecated
        Use Access Keys instead, see "access_key" argument.
    """
    _common.validate_argument_types([
        (username, 'username', str),
        (password, 'password', str),
        (access_key, 'access_key', str),
        (url, 'url', str),
        (directory, 'directory', str),
        (ignore_ssl_errors, 'ignore_ssl_errors', bool),
        (proxy, 'proxy', str),
        (credentials_file, 'credentials_file', str),
        # Note: Although auth_token is no longer a supported authentication method for non-Seeq Data Lab scenarios,
        # it is still used by Seeq Data Lab code to log the user in.
        (auth_token, 'auth_token', str),
        (force, 'force', bool),
        (quiet, 'quiet', bool),
        (status, 'status', _common.Status)
    ])

    status = Status.validate(status, quiet)

    try:
        _login(username, password, access_key, url, directory, ignore_ssl_errors, proxy,
               credentials_file, auth_token, force, status, auth_provider)
    except (RuntimeError, ValueError, TypeError) as e:
        status.update(str(e), Status.FAILURE)
        raise

    user_string = user.username
    user_profile = ''
    if user.first_name:
        user_profile = user.first_name
    if user.last_name:
        user_profile += ' ' + user.last_name
    if user.is_admin:
        user_profile += ' [Admin]'
    if len(user_profile) > 0:
        user_string += ' (%s)' % user_profile.strip()

    status.update(f'Logged in to <strong>{_config.get_seeq_url()}</strong> '
                  f'successfully as <strong>{user_string}</strong>.\n'
                  f'Seeq Server Version: <strong>{spy.server_version}</strong>\n'
                  f'Seeq Python Module Version: <strong>{seeq.__version__}</strong>',
                  Status.SUCCESS)


def _login(username, password, access_key, url, directory, ignore_ssl_errors, proxy, credentials_file, auth_token,
           force, status, auth_provider):
    if auth_provider is not None:
        raise ValueError('"auth_provider" argument has been renamed to "directory"')

    if access_key and username:
        raise ValueError('"username" argument must be omitted when supplying "access_key" argument')

    if access_key and directory != 'Seeq':
        raise ValueError('"directory" argument must be omitted when supplying "access_key" argument')

    global client, user, supported_units, corporate_folder

    try:
        if force or not client:
            # Clear out any global state before attempting to log in
            _clear_login_state(quiet=True, status=None)

            if url:
                _config.set_seeq_url(_url.cleanse_url(url))

            _client_login(auth_token, credentials_file, directory, ignore_ssl_errors, password, proxy, status,
                          _config.get_api_url(), username, access_key)

        system_api = SystemApi(client)
        server_status = system_api.get_server_status()  # type: ServerStatusOutputV1
        spy.server_version = server_status.version
        server_major, server_minor, server_patch = validate_seeq_server_version(
            status,
            # We allow a version mismatch here because in the case of Data Lab, this login call happens when the
            # kernel is initialized, and if an exception is thrown, it is not seen by the user. Then subsequent calls to
            # other functions like spy.search() fail with a "not logged in" error instead of failing with a version
            # mismatch error. That's confusing. So instead, we just warn during login (but succeed) and then fail later
            # when other functions are called.
            allow_version_mismatch=True)

        if server_minor >= 46:
            validate_data_lab_license()

        users_api = UsersApi(client)
        user = users_api.get_me()  # type: UserOutputV1
        spy.user = user

        folders_api = FoldersApi(client)

        # noinspection PyBroadException
        try:
            corporate_folder = folders_api.get_folder(folder_id='corporate')
        except BaseException:
            # This can happen in cases where the user does not have access rights to the Corporate folder
            corporate_folder = None

    except BaseException:
        logout(quiet=True)
        raise


def _client_login(auth_token, credentials_file, directory, ignore_ssl_errors, password, proxy, status, api_client_url,
                  username, access_key):
    # Annoying warnings are printed to stderr if connections fail
    logging.getLogger("requests").setLevel(logging.FATAL)
    logging.getLogger("urllib3").setLevel(logging.FATAL)
    urllib3.disable_warnings()

    cert_file = _config.get_seeq_cert_path()
    if cert_file and os.path.exists(cert_file):
        Configuration().set_certificate_path(cert_file)

    key_file = _config.get_seeq_key_path()
    if key_file and os.path.exists(key_file):
        Configuration().key_file = key_file

    Configuration().verify_ssl = not ignore_ssl_errors

    if proxy == AUTOMATIC_PROXY_DETECTION:
        if api_client_url.startswith('https') and 'HTTPS_PROXY' in os.environ:
            Configuration().proxy = os.environ['HTTPS_PROXY']
        elif 'HTTP_PROXY' in os.environ:
            Configuration().proxy = os.environ['HTTP_PROXY']
    elif proxy is not None:
        Configuration().proxy = proxy

    _client = ApiClient(api_client_url)
    _client.set_default_header('x-sq-origin', 'Data Lab')

    if auth_token:
        if username or password or credentials_file:
            raise ValueError('username, password and/or credentials_file cannot be provided along with auth_token')

        _client.auth_token = auth_token
    else:
        auth_api = AuthApi(_client)
        auth_input = AuthInputV1()

        if access_key:
            username = access_key

        if credentials_file:
            if username is not None or password is not None or access_key is not None:
                raise ValueError('If credentials_file is specified, username, access_key and password must be omitted')

            if not os.path.exists(credentials_file):
                repo_root_dir = system.get_repo_root_dir(windows_long_path=False)
                if repo_root_dir:
                    agent_key_file = os.path.join(system.get_repo_root_dir(windows_long_path=False), 'sq-run-data-dir',
                                                  'keys', 'agent.key')
                    if os.path.exists(agent_key_file):
                        # This is a hack to make it easier to run the Documentation notebooks without having to create a
                        # credentials file. It was too easy to forget to delete the credentials file when you created
                        # it, and it would wind up in the distribution.
                        credentials_file = agent_key_file

            if not os.path.exists(credentials_file):
                raise ValueError(f'credentials_file "{credentials_file}" not found')

            try:
                with open(credentials_file) as f:
                    lines = f.readlines()
            except Exception as e:
                raise RuntimeError('Could not read credentials_file "%s": %s' % (credentials_file, e))

            if len(lines) < 2:
                raise RuntimeError('credentials_file "%s" must have two lines: username then password')

            username = lines[0].strip()
            password = lines[1].strip()

        if not username or not password:
            if access_key:
                raise ValueError('Both access_key and password must be supplied')
            else:
                raise ValueError('Both username and password must be supplied')

        auth_input.username = username
        auth_input.password = password

        status.update('Logging in to <strong>%s</strong> as <strong>%s</strong>' % (
            api_client_url, username), Status.RUNNING)

        directories = dict()
        try:
            auth_providers_output = auth_api.get_auth_providers()  # type: AuthProvidersOutputV1
        except MaxRetryError as e:
            if isinstance(e.reason, SSLError):
                raise RuntimeError(f'SSL certificate error. If you trust your network, you can add '
                                   f'the spy.login(ignore_ssl_errors=True) argument.\n\nMore info:\n{e.reason}')

            raise RuntimeError(
                '"%s" could not be reached. Is the server or network down?\n%s' % (api_client_url, e))

        for datasource_output in auth_providers_output.auth_providers:  # type: DatasourceOutputV1
            directories[datasource_output.name] = datasource_output

        if directory not in directories:
            raise RuntimeError('directory "%s" not recognized. Possible directory(s) for this server: %s' %
                               (directory, ', '.join(directories.keys())))

        datasource_output = directories[directory]
        auth_input.auth_provider_class = datasource_output.datasource_class
        auth_input.auth_provider_id = datasource_output.datasource_id

        try:
            auth_api.login(body=auth_input)
        except ApiException as e:
            if e.status == 401:
                if access_key:
                    seeq_module_major, _, _, _ = get_module_version_tuple()
                    if seeq_module_major < 50:
                        raise RuntimeError(
                            f'Access Key "{access_key}" is not valid. Log out of the Seeq user interface  (if logged '
                            'in) and then log back in again to reset Access Key validity. If you are still seeing '
                            'this error after doing so, make sure the "access_key" and "password" arguments are '
                            'correct and match an Access Key that you have created in the Seeq user interface.'
                        )
                    else:
                        raise RuntimeError(
                            f'Access Key "{access_key}" is not valid. Log in to the Seeq user interface to reset its '
                            'validity. If you are still seeing this error after doing so, make sure the "access_key" '
                            'and "password" arguments are correct and match an Access Key that you have created in '
                            'the Seeq user interface.'
                        )
                else:
                    raise RuntimeError(
                        f'"{auth_input.username}" could not be logged in with supplied credentials, check username and '
                        'password.')
            else:
                raise
        except MaxRetryError as e:
            raise RuntimeError(
                f'"{api_client_url}" could not be reached. Is the server or network down?\n{e}')
        except Exception as e:
            raise RuntimeError(
                f'Could not connect to Seeq\'s API at {api_client_url} with login "{auth_input.username}".\n{e}')

    # Now that we have succeeded, set all the global variables
    global client, https_verify_ssl, https_key_file, https_cert_file
    client = _client
    spy.client = client

    https_verify_ssl = not ignore_ssl_errors
    https_key_file = key_file
    https_cert_file = cert_file


def logout(quiet=False, status=None):
    """
    Logs you out of your current session.

    Parameters
    ----------

    quiet : bool, default False
        If True, suppresses progress output.

    status : spy.Status, optional
        If supplied, this Status object will be updated as the command
        progresses.
    """

    _clear_login_state(quiet, status, call_logout=True)


def _clear_login_state(quiet, status, call_logout=False):
    """
    Clear out any global state and optionally logs you out of your current session.

    Parameters
    ----------

    quiet : bool
        If True, suppresses progress output.

    status : spy.Status, optional
        If supplied, this Status object will be updated as the command
        progresses.

    call_logout : bool
        If True, auth_api.logout is called otherwise just the global state is cleared
    """

    status = Status.validate(status, quiet)
    global client, supported_units, user, https_verify_ssl, https_key_file, https_cert_file

    if client is None:
        status.update('No action taken because you are not currently logged in.', Status.FAILURE)
    else:
        if call_logout:
            auth_api = AuthApi(client)
            auth_api.logout()

        client.logout()
        client = None
        spy.client = None

    user = None
    spy.user = None
    supported_units = None
    https_verify_ssl = True
    https_key_file = None
    https_cert_file = None

    status.update('Logged out.', Status.SUCCESS)


def find_user(query):
    """
    Finds a user by using Seeq's user/group autocomplete functionality. Must result in exactly one match or a
    RuntimeError is raised.
    :param query: A user/group fragment to use for the search
    :return: The identity of the matching user.
    :rtype: UserOutputV1
    """
    users_api = UsersApi(client)

    if _common.is_guid(query):
        user_id = query
    else:
        identity_preview_list = users_api.autocomplete_users_and_groups(query=query)  # type: IdentityPreviewListV1
        if len(identity_preview_list.items) == 0:
            raise RuntimeError('User "%s" not found' % query)
        if len(identity_preview_list.items) > 1:
            raise RuntimeError('Multiple users found that match "%s":\n%s' % (
                query, '\n'.join([('%s (%s)' % (u.username, u.id)) for u in identity_preview_list.items])))

        user_id = identity_preview_list.items[0].id

    return users_api.get_user(id=user_id)


def get_user_timezone():
    # noinspection PyBroadException
    try:
        workbench_dict = json.loads(user.workbench)
        return workbench_dict['state']['stores']['sqWorkbenchStore']['userTimeZone']
    except BaseException:
        # This can happen if the user has never logged in interactively (e.g., agent_api_key)
        return 'UTC'


def get_module_version_tuple():
    match = re.match(r'^(\d+)\.(\d+)\.(\d+)\.(\d+)', seeq.__version__)
    seeq_module_major = int(match.group(1))
    seeq_module_minor = int(match.group(2))
    seeq_module_patch = int(match.group(3))
    seeq_module_functional = int(match.group(4))
    return seeq_module_major, seeq_module_minor, seeq_module_patch, seeq_module_functional


def get_server_version_tuple():
    match = re.match(r'^R?(?:\d+\.)?(\d+)\.(\d+)\.(\d+)(-v\w+)?(-[-\w]+)?', spy.server_version)
    seeq_server_major = int(match.group(1))
    seeq_server_minor = int(match.group(2))
    seeq_server_patch = int(match.group(3))
    return seeq_server_major, seeq_server_minor, seeq_server_patch


def validate_seeq_server_version(status: Status, allow_version_mismatch=False):
    seeq_module_major, seeq_module_minor, _, _ = get_module_version_tuple()
    seeq_server_major, seeq_server_minor, seeq_server_patch = get_server_version_tuple()

    # The old versioning scheme is like 0.49.3 whereas the new scheme is like 50.1.8
    # See https://seeq.atlassian.net/wiki/spaces/SQ/pages/947225963/Seeq+Versioning+Simplification
    using_old_versioning_scheme = (seeq_server_major == 0)

    message = None

    if using_old_versioning_scheme:
        if seeq_module_major != seeq_server_major or \
                seeq_module_minor != seeq_server_minor:
            message = (f'The major/minor version of the seeq module ({seeq_module_major}.{seeq_module_minor}) '
                       f'does not match the major/minor version of the Seeq Server you are connected to '
                       f'({seeq_server_major}.{seeq_server_minor}) and is incompatible.')
    else:
        if seeq_module_major != seeq_server_major:
            message = (f'The major version of the seeq module ({seeq_module_major}) '
                       f'does not match the major version of the Seeq Server you are connected to '
                       f'({seeq_server_major}) and is incompatible.')

    if message:
        compatible_module_folder = find_compatible_module()
        if not compatible_module_folder:
            message += (f'\n\nIt is recommended that you issue the following PIP command to '
                        f'install a compatible version of the seeq module:\n')
            if using_old_versioning_scheme:
                message += f'pip install -U seeq~={seeq_server_major}.{seeq_server_minor}.{seeq_server_patch}'
            else:
                message += f'pip install -U seeq~={seeq_server_major}.{seeq_server_minor}'
        else:
            message += (f'\n\nA compatible module was found here: {compatible_module_folder}\n'
                        f'Consider uninstalling your project-specific seeq module by issuing the following PIP '
                        f'command:\n'
                        f'pip uninstall -y seeq')

        if allow_version_mismatch or _config.options.allow_version_mismatch:
            status.warn(message)
        else:
            raise RuntimeError(message)

    return seeq_server_major, seeq_server_minor, seeq_server_patch


def find_compatible_module():
    """
    Uses pkg_resources tools to look for a seeq module that is compatible with the version of Seeq Server we're
    connected to. This function is useful in Seeq Data Lab scenarios where the user has installed a "private" version of
    the seeq module (presumably to get a bugfix for SPy) but Seeq Server and Seeq Data Lab have been upgraded in the
    meantime and the user's best course of action is to remove the private version and resume using the "built-in"
    version, which will update with Seeq Data Lab as it is upgraded.
    :return: Path to a compatible module, None if a compatible module is not found.
    """
    seeq_module_major, seeq_module_minor, _, _ = get_module_version_tuple()
    seeq_server_major, seeq_server_minor, seeq_server_patch = get_server_version_tuple()

    pkg_env = pkg_resources.Environment()
    seeq_modules = pkg_env['seeq']
    for seeq_module in seeq_modules:  # type: pkg_resources.Distribution
        if seeq_server_major >= 50 and seeq_module.version.startswith(f'{seeq_server_major}.'):
            return seeq_module.location
        elif seeq_server_major < 50 and seeq_module.version.startswith(f'{seeq_server_major}.{seeq_server_minor}.'):
            return seeq_module.location

    return None


def validate_data_lab_license():
    global client
    system_api = SystemApi(client)
    license_status_output = system_api.get_license()  # type: LicenseStatusOutputV1
    for additional_feature in license_status_output.additional_features:  # type: LicensedFeatureStatusOutputV1
        if additional_feature.name == 'Data_Lab':
            if additional_feature.validity == 'Valid':
                return

            raise RuntimeError(f'Seeq Data Lab license is "{additional_feature.validity}", could not log in. Contact '
                               'your administrator or speak with Seeq Support at support@seeq.com.')

    raise RuntimeError('Seeq Data Lab is not licensed for this server, could not log in. Contact your administrator '
                       'or speak with Seeq Support at support@seeq.com.')


def validate_login(status: Status):
    global client
    if client is None:
        raise RuntimeError('Not logged in. Execute spy.login() before calling this function.')

    validate_seeq_server_version(status)


def localize_datetime(dt, default_timezone=None):
    if not dt.tz:
        dt = dt.tz_localize(default_timezone if default_timezone else get_user_timezone())
    return dt


def parse_datetime_with_timezone(dt, nanoseconds_multiplier=1000000, default_timezone=None):
    # type: (str, int, object) -> pd.Timestamp

    if isinstance(dt, pd.Timestamp):
        return localize_datetime(dt, default_timezone=default_timezone)

    if isinstance(dt, (float, int)):
        # Assume it's in milliseconds because it came from the front-end
        return pd.Timestamp(int(float(dt) * nanoseconds_multiplier)).tz_localize('UTC')

    match = re.fullmatch(r'^([^\[\]]+)(\[(.+)\])?$', dt)
    if not match or not match.group(3):
        return localize_datetime(pd.to_datetime(dt))

    datetime_part = match.group(1)
    tz_part = match.group(3)
    return pd.Timestamp(datetime_part).tz_localize(tz_part)


# The list of units that come back from system_api.get_supported_units() is a curated set of compound units that are
# meant to be human-friendly in Formula tool help. But it's the only thing available in the API to figure out what
# base units are supported, so we have to split the compound unit string into its components. I.e., S/cm³ gets split
# into S and cm so that we can add those two "base" units to the set we use to determine validity.
UNITS_SPLIT_REGEX = r'[/*²³·]'


def is_valid_unit(unit):
    """
    Returns True if the supplied unit will be recognized by the Seeq calculation engine. This can be an important
    function to use if you are attempting to supply a "Value Unit Of Measure" property on a Signal or a "Unit Of
    Measure" property on a Scalar.
    :param unit: The unit of measure for which to assess validity
    :return: True if unit is valid, False if not
    """
    global supported_units, client
    if not supported_units:
        system_api = SystemApi(client)
        support_units_output = system_api.get_supported_units()  # type: SupportedUnitsOutputV1

        supported_units = set()
        for supported_unit_family in support_units_output.units.values():
            for supported_unit in supported_unit_family:
                unit_parts = re.split(UNITS_SPLIT_REGEX, supported_unit)
                supported_units.update([u for u in unit_parts if len(u) > 0])

    unit_parts = re.split(UNITS_SPLIT_REGEX, unit)
    for unit_part in [u for u in unit_parts if len(u) > 0]:
        if unit_part not in supported_units:
            return False

    return True


def pull_image(url):
    return requests.get(url, headers={
        "Accept": "application/vnd.seeq.v1+json",
        "x-sq-auth": client.auth_token
    }, verify=https_verify_ssl).content



# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 50.4.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class WorkbookInputV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'branch_from': 'str',
        'description': 'str',
        'folder_id': 'str',
        'name': 'str',
        'owner_id': 'str',
        'type': 'str'
    }

    attribute_map = {
        'branch_from': 'branchFrom',
        'description': 'description',
        'folder_id': 'folderId',
        'name': 'name',
        'owner_id': 'ownerId',
        'type': 'type'
    }

    def __init__(self, branch_from=None, description=None, folder_id=None, name=None, owner_id=None, type=None):
        """
        WorkbookInputV1 - a model defined in Swagger
        """

        self._branch_from = None
        self._description = None
        self._folder_id = None
        self._name = None
        self._owner_id = None
        self._type = None

        if branch_from is not None:
          self.branch_from = branch_from
        if description is not None:
          self.description = description
        if folder_id is not None:
          self.folder_id = folder_id
        if name is not None:
          self.name = name
        if owner_id is not None:
          self.owner_id = owner_id
        if type is not None:
          self.type = type

    @property
    def branch_from(self):
        """
        Gets the branch_from of this WorkbookInputV1.
        Create a new workbook by duplicating the contents and history of the workbook with the specified ID. When null, no branching will occur; resulting workbook will be empty.

        :return: The branch_from of this WorkbookInputV1.
        :rtype: str
        """
        return self._branch_from

    @branch_from.setter
    def branch_from(self, branch_from):
        """
        Sets the branch_from of this WorkbookInputV1.
        Create a new workbook by duplicating the contents and history of the workbook with the specified ID. When null, no branching will occur; resulting workbook will be empty.

        :param branch_from: The branch_from of this WorkbookInputV1.
        :type: str
        """

        self._branch_from = branch_from

    @property
    def description(self):
        """
        Gets the description of this WorkbookInputV1.
        Clarifying information or other plain language description of this item. An input of just whitespace is equivalent to a null input.

        :return: The description of this WorkbookInputV1.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this WorkbookInputV1.
        Clarifying information or other plain language description of this item. An input of just whitespace is equivalent to a null input.

        :param description: The description of this WorkbookInputV1.
        :type: str
        """

        self._description = description

    @property
    def folder_id(self):
        """
        Gets the folder_id of this WorkbookInputV1.
        The id of the folder to place the new workbook into.If null, the workbook will be created in the user's home folder.

        :return: The folder_id of this WorkbookInputV1.
        :rtype: str
        """
        return self._folder_id

    @folder_id.setter
    def folder_id(self, folder_id):
        """
        Sets the folder_id of this WorkbookInputV1.
        The id of the folder to place the new workbook into.If null, the workbook will be created in the user's home folder.

        :param folder_id: The folder_id of this WorkbookInputV1.
        :type: str
        """

        self._folder_id = folder_id

    @property
    def name(self):
        """
        Gets the name of this WorkbookInputV1.
        Human readable name. Required during creation. An input of just whitespaces is equivalent to a null input.

        :return: The name of this WorkbookInputV1.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this WorkbookInputV1.
        Human readable name. Required during creation. An input of just whitespaces is equivalent to a null input.

        :param name: The name of this WorkbookInputV1.
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")

        self._name = name

    @property
    def owner_id(self):
        """
        Gets the owner_id of this WorkbookInputV1.
        The ID of the User that owns this workbook. If omitted when creating a new Workbook, the authenticated user is used by default. Only administrators can set this value.

        :return: The owner_id of this WorkbookInputV1.
        :rtype: str
        """
        return self._owner_id

    @owner_id.setter
    def owner_id(self, owner_id):
        """
        Sets the owner_id of this WorkbookInputV1.
        The ID of the User that owns this workbook. If omitted when creating a new Workbook, the authenticated user is used by default. Only administrators can set this value.

        :param owner_id: The owner_id of this WorkbookInputV1.
        :type: str
        """

        self._owner_id = owner_id

    @property
    def type(self):
        """
        Gets the type of this WorkbookInputV1.
        The type of workbook, either 'Analysis' or 'Topic'. Defaults to 'Analysis'.

        :return: The type of this WorkbookInputV1.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the type of this WorkbookInputV1.
        The type of workbook, either 'Analysis' or 'Topic'. Defaults to 'Analysis'.

        :param type: The type of this WorkbookInputV1.
        :type: str
        """

        self._type = type

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, WorkbookInputV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 50.4.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class JobOutputV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'configuration': 'dict(str, object)',
        'document_id': 'str',
        'document_name': 'str',
        'duration': 'int',
        'group': 'str',
        'id': 'str',
        'next_run_time': 'str',
        'previous_run_time': 'str',
        'status_message': 'str',
        'topic_id': 'str',
        'topic_name': 'str'
    }

    attribute_map = {
        'configuration': 'configuration',
        'document_id': 'documentId',
        'document_name': 'documentName',
        'duration': 'duration',
        'group': 'group',
        'id': 'id',
        'next_run_time': 'nextRunTime',
        'previous_run_time': 'previousRunTime',
        'status_message': 'statusMessage',
        'topic_id': 'topicId',
        'topic_name': 'topicName'
    }

    def __init__(self, configuration=None, document_id=None, document_name=None, duration=None, group=None, id=None, next_run_time=None, previous_run_time=None, status_message=None, topic_id=None, topic_name=None):
        """
        JobOutputV1 - a model defined in Swagger
        """

        self._configuration = None
        self._document_id = None
        self._document_name = None
        self._duration = None
        self._group = None
        self._id = None
        self._next_run_time = None
        self._previous_run_time = None
        self._status_message = None
        self._topic_id = None
        self._topic_name = None

        if configuration is not None:
          self.configuration = configuration
        if document_id is not None:
          self.document_id = document_id
        if document_name is not None:
          self.document_name = document_name
        if duration is not None:
          self.duration = duration
        if group is not None:
          self.group = group
        if id is not None:
          self.id = id
        if next_run_time is not None:
          self.next_run_time = next_run_time
        if previous_run_time is not None:
          self.previous_run_time = previous_run_time
        if status_message is not None:
          self.status_message = status_message
        if topic_id is not None:
          self.topic_id = topic_id
        if topic_name is not None:
          self.topic_name = topic_name

    @property
    def configuration(self):
        """
        Gets the configuration of this JobOutputV1.
        The configuration for the job

        :return: The configuration of this JobOutputV1.
        :rtype: dict(str, object)
        """
        return self._configuration

    @configuration.setter
    def configuration(self, configuration):
        """
        Sets the configuration of this JobOutputV1.
        The configuration for the job

        :param configuration: The configuration of this JobOutputV1.
        :type: dict(str, object)
        """

        self._configuration = configuration

    @property
    def document_id(self):
        """
        Gets the document_id of this JobOutputV1.
        The ID of the topic document, if supplied, that requested this job

        :return: The document_id of this JobOutputV1.
        :rtype: str
        """
        return self._document_id

    @document_id.setter
    def document_id(self, document_id):
        """
        Sets the document_id of this JobOutputV1.
        The ID of the topic document, if supplied, that requested this job

        :param document_id: The document_id of this JobOutputV1.
        :type: str
        """

        self._document_id = document_id

    @property
    def document_name(self):
        """
        Gets the document_name of this JobOutputV1.
        The name of the topic document, if supplied, which requested this job

        :return: The document_name of this JobOutputV1.
        :rtype: str
        """
        return self._document_name

    @document_name.setter
    def document_name(self, document_name):
        """
        Sets the document_name of this JobOutputV1.
        The name of the topic document, if supplied, which requested this job

        :param document_name: The document_name of this JobOutputV1.
        :type: str
        """

        self._document_name = document_name

    @property
    def duration(self):
        """
        Gets the duration of this JobOutputV1.
        The amount of time, in nanoseconds, that the job took for its last execution

        :return: The duration of this JobOutputV1.
        :rtype: int
        """
        return self._duration

    @duration.setter
    def duration(self, duration):
        """
        Sets the duration of this JobOutputV1.
        The amount of time, in nanoseconds, that the job took for its last execution

        :param duration: The duration of this JobOutputV1.
        :type: int
        """

        self._duration = duration

    @property
    def group(self):
        """
        Gets the group of this JobOutputV1.
        The group to which the job belongs

        :return: The group of this JobOutputV1.
        :rtype: str
        """
        return self._group

    @group.setter
    def group(self, group):
        """
        Sets the group of this JobOutputV1.
        The group to which the job belongs

        :param group: The group of this JobOutputV1.
        :type: str
        """

        self._group = group

    @property
    def id(self):
        """
        Gets the id of this JobOutputV1.
        The id of the job

        :return: The id of this JobOutputV1.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this JobOutputV1.
        The id of the job

        :param id: The id of this JobOutputV1.
        :type: str
        """

        self._id = id

    @property
    def next_run_time(self):
        """
        Gets the next_run_time of this JobOutputV1.
        The next time, as an ISO timestamp, the job is scheduled to run. Empty if it is not scheduled

        :return: The next_run_time of this JobOutputV1.
        :rtype: str
        """
        return self._next_run_time

    @next_run_time.setter
    def next_run_time(self, next_run_time):
        """
        Sets the next_run_time of this JobOutputV1.
        The next time, as an ISO timestamp, the job is scheduled to run. Empty if it is not scheduled

        :param next_run_time: The next_run_time of this JobOutputV1.
        :type: str
        """

        self._next_run_time = next_run_time

    @property
    def previous_run_time(self):
        """
        Gets the previous_run_time of this JobOutputV1.
        The previous time, as an ISO timestamp, at which the job was run. Empty if it has not been run

        :return: The previous_run_time of this JobOutputV1.
        :rtype: str
        """
        return self._previous_run_time

    @previous_run_time.setter
    def previous_run_time(self, previous_run_time):
        """
        Sets the previous_run_time of this JobOutputV1.
        The previous time, as an ISO timestamp, at which the job was run. Empty if it has not been run

        :param previous_run_time: The previous_run_time of this JobOutputV1.
        :type: str
        """

        self._previous_run_time = previous_run_time

    @property
    def status_message(self):
        """
        Gets the status_message of this JobOutputV1.
        A plain language status message with information about any issues that may have been encountered during an operation. Null if the status message has not been set.

        :return: The status_message of this JobOutputV1.
        :rtype: str
        """
        return self._status_message

    @status_message.setter
    def status_message(self, status_message):
        """
        Sets the status_message of this JobOutputV1.
        A plain language status message with information about any issues that may have been encountered during an operation. Null if the status message has not been set.

        :param status_message: The status_message of this JobOutputV1.
        :type: str
        """

        self._status_message = status_message

    @property
    def topic_id(self):
        """
        Gets the topic_id of this JobOutputV1.
        The ID of the Topic, if supplied, that requested this job

        :return: The topic_id of this JobOutputV1.
        :rtype: str
        """
        return self._topic_id

    @topic_id.setter
    def topic_id(self, topic_id):
        """
        Sets the topic_id of this JobOutputV1.
        The ID of the Topic, if supplied, that requested this job

        :param topic_id: The topic_id of this JobOutputV1.
        :type: str
        """

        self._topic_id = topic_id

    @property
    def topic_name(self):
        """
        Gets the topic_name of this JobOutputV1.
        The name of the topic, if supplied, that requested this job

        :return: The topic_name of this JobOutputV1.
        :rtype: str
        """
        return self._topic_name

    @topic_name.setter
    def topic_name(self, topic_name):
        """
        Sets the topic_name of this JobOutputV1.
        The name of the topic, if supplied, that requested this job

        :param topic_name: The topic_name of this JobOutputV1.
        :type: str
        """

        self._topic_name = topic_name

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, JobOutputV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



from ._search import search
from ._pull import pull
from ._push import push
from ._load import load
from ._save import save

from ._data import CalculatedSignal, CalculatedCondition, CalculatedScalar, Chart, Datasource, TableDatasource, \
    StoredSignal, StoredCondition, ThresholdMetric
from ._folder import Folder, SHARED, CORPORATE, ALL, SYNTHETIC_FOLDERS
from ._item import Item, options
from ._user import User, UserGroup, ORIGINAL_OWNER, FORCE_ME_AS_OWNER
from ._workbook import Workbook, Analysis, Topic
from ._worksheet import Worksheet, AnalysisWorksheet, TopicDocument
from ._workstep import AnalysisWorkstep
from ._annotation import Annotation, Report, Journal
from ._content import DateRange, Content

__all__ = ['search',
           'pull',
           'push',
           'load',
           'save',
           'Workbook',
           'Analysis',
           'Topic',
           'DateRange',
           'Content',
           'Annotation',
           'Report',
           'Journal',
           'Worksheet',
           'AnalysisWorksheet',
           'TopicDocument',
           'options',
           'ORIGINAL_OWNER', 'FORCE_ME_AS_OWNER',
           'SHARED', 'CORPORATE', 'ALL', 'SYNTHETIC_FOLDERS']

Item.available_types = {
    'CalculatedCondition': CalculatedCondition,
    'CalculatedScalar': CalculatedScalar,
    'CalculatedSignal': CalculatedSignal,
    'Chart': Chart,
    'Datasource': Datasource,
    'Folder': Folder,
    'StoredCondition': StoredCondition,
    'StoredSignal': StoredSignal,
    'TableDatasource': TableDatasource,
    'ThresholdMetric': ThresholdMetric,
    'Workbook': Workbook,
    'Worksheet': Worksheet,
    'Workstep': AnalysisWorkstep,
    'User': User,
    'UserGroup': UserGroup,
    'Annotation': Annotation,
    'Report': Report,
    'Journal': Journal,
    'DateRange': DateRange,
    'Content': Content
}



from .._common import \
    is_guid, \
    time_abbreviation_to_ms_multiplier, \
    parse_str_time_to_ms

from .._url import \
    get_workbook_id_from_url, \
    get_worksheet_id_from_url, \
    get_workstep_id_from_url

from .._login import \
    is_valid_unit

from ..workbooks._worksheet import get_analysis_worksheet_from_url

from .._config import get_data_lab_project_url, get_data_lab_project_id

__all__ = ['get_workbook_id_from_url',
           'get_worksheet_id_from_url',
           'get_workstep_id_from_url',
           'get_analysis_worksheet_from_url',
           'is_guid',
           'is_valid_unit',
           'time_abbreviation_to_ms_multiplier',
           'parse_str_time_to_ms',
           'get_data_lab_project_url',
           'get_data_lab_project_id']



# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 50.4.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class DatasourcePreviewV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'datasource_class': 'str',
        'datasource_id': 'str',
        'href': 'str',
        'id': 'str',
        'is_archived': 'bool',
        'is_redacted': 'bool',
        'name': 'str',
        'type': 'str'
    }

    attribute_map = {
        'datasource_class': 'datasourceClass',
        'datasource_id': 'datasourceId',
        'href': 'href',
        'id': 'id',
        'is_archived': 'isArchived',
        'is_redacted': 'isRedacted',
        'name': 'name',
        'type': 'type'
    }

    def __init__(self, datasource_class=None, datasource_id=None, href=None, id=None, is_archived=False, is_redacted=False, name=None, type=None):
        """
        DatasourcePreviewV1 - a model defined in Swagger
        """

        self._datasource_class = None
        self._datasource_id = None
        self._href = None
        self._id = None
        self._is_archived = None
        self._is_redacted = None
        self._name = None
        self._type = None

        if datasource_class is not None:
          self.datasource_class = datasource_class
        if datasource_id is not None:
          self.datasource_id = datasource_id
        if href is not None:
          self.href = href
        if id is not None:
          self.id = id
        if is_archived is not None:
          self.is_archived = is_archived
        if is_redacted is not None:
          self.is_redacted = is_redacted
        if name is not None:
          self.name = name
        if type is not None:
          self.type = type

    @property
    def datasource_class(self):
        """
        Gets the datasource_class of this DatasourcePreviewV1.
        The class of the datasource

        :return: The datasource_class of this DatasourcePreviewV1.
        :rtype: str
        """
        return self._datasource_class

    @datasource_class.setter
    def datasource_class(self, datasource_class):
        """
        Sets the datasource_class of this DatasourcePreviewV1.
        The class of the datasource

        :param datasource_class: The datasource_class of this DatasourcePreviewV1.
        :type: str
        """

        self._datasource_class = datasource_class

    @property
    def datasource_id(self):
        """
        Gets the datasource_id of this DatasourcePreviewV1.
        The ID of the datasource

        :return: The datasource_id of this DatasourcePreviewV1.
        :rtype: str
        """
        return self._datasource_id

    @datasource_id.setter
    def datasource_id(self, datasource_id):
        """
        Sets the datasource_id of this DatasourcePreviewV1.
        The ID of the datasource

        :param datasource_id: The datasource_id of this DatasourcePreviewV1.
        :type: str
        """

        self._datasource_id = datasource_id

    @property
    def href(self):
        """
        Gets the href of this DatasourcePreviewV1.
        The href that can be used to interact with the item

        :return: The href of this DatasourcePreviewV1.
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """
        Sets the href of this DatasourcePreviewV1.
        The href that can be used to interact with the item

        :param href: The href of this DatasourcePreviewV1.
        :type: str
        """
        if href is None:
            raise ValueError("Invalid value for `href`, must not be `None`")

        self._href = href

    @property
    def id(self):
        """
        Gets the id of this DatasourcePreviewV1.
        The ID that can be used to interact with the item

        :return: The id of this DatasourcePreviewV1.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this DatasourcePreviewV1.
        The ID that can be used to interact with the item

        :param id: The id of this DatasourcePreviewV1.
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")

        self._id = id

    @property
    def is_archived(self):
        """
        Gets the is_archived of this DatasourcePreviewV1.
        Whether item is archived

        :return: The is_archived of this DatasourcePreviewV1.
        :rtype: bool
        """
        return self._is_archived

    @is_archived.setter
    def is_archived(self, is_archived):
        """
        Sets the is_archived of this DatasourcePreviewV1.
        Whether item is archived

        :param is_archived: The is_archived of this DatasourcePreviewV1.
        :type: bool
        """

        self._is_archived = is_archived

    @property
    def is_redacted(self):
        """
        Gets the is_redacted of this DatasourcePreviewV1.
        Whether item is redacted

        :return: The is_redacted of this DatasourcePreviewV1.
        :rtype: bool
        """
        return self._is_redacted

    @is_redacted.setter
    def is_redacted(self, is_redacted):
        """
        Sets the is_redacted of this DatasourcePreviewV1.
        Whether item is redacted

        :param is_redacted: The is_redacted of this DatasourcePreviewV1.
        :type: bool
        """

        self._is_redacted = is_redacted

    @property
    def name(self):
        """
        Gets the name of this DatasourcePreviewV1.
        The human readable name

        :return: The name of this DatasourcePreviewV1.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this DatasourcePreviewV1.
        The human readable name

        :param name: The name of this DatasourcePreviewV1.
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")

        self._name = name

    @property
    def type(self):
        """
        Gets the type of this DatasourcePreviewV1.
        The type of the item

        :return: The type of this DatasourcePreviewV1.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the type of this DatasourcePreviewV1.
        The type of the item

        :param type: The type of this DatasourcePreviewV1.
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")

        self._type = type

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, DatasourcePreviewV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
